@@ -1,6 +1,6 @@
 /**
  * This file has no copyright assigned and is placed in the Public Domain.
- * This file is part of the w64 mingw-runtime package.
+ * This file is part of the mingw-w64 runtime package.
  * No warranty is given; refer to the file DISCLAIMER.PD within this package.
  */
 
@@ -32,7 +32,9 @@ extern char *** __MINGW_IMP_SYMBOL(__initenv);
 #endif
 
 /* Hack, for bug in ld.  Will be removed soon.  */
+#if defined(__GNUC__)
 #define __ImageBase __MINGW_LSYMBOL(_image_base__)
+#endif
 /* This symbol is defined by ld.  */
 extern IMAGE_DOS_HEADER __ImageBase;
 
@@ -68,7 +70,7 @@ extern int mingw_app_type;
 
 HINSTANCE __mingw_winmain_hInstance;
 _TCHAR *__mingw_winmain_lpCmdLine;
-DWORD __mingw_winmain_nShowCmd;
+DWORD __mingw_winmain_nShowCmd = SW_SHOWDEFAULT;
 
 static int argc;
 extern void __main(void);
@@ -97,12 +99,29 @@ static void duplicate_ppstrings (int ac, char ***av);
 
 static int __cdecl pre_c_init (void);
 static void __cdecl pre_cpp_init (void);
-static void __cdecl __mingw_prepare_except_for_msvcr80_and_higher (void);
 _CRTALLOC(".CRT$XIAA") _PIFV mingw_pcinit = pre_c_init;
 _CRTALLOC(".CRT$XCAA") _PVFV mingw_pcppinit = pre_cpp_init;
 
 extern int _MINGW_INSTALL_DEBUG_MATHERR;
 
+#ifdef __MINGW_SHOW_INVALID_PARAMETER_EXCEPTION
+#define __UNUSED_PARAM_1(x) x
+#else
+#define __UNUSED_PARAM_1	__UNUSED_PARAM
+#endif
+static void
+__mingw_invalidParameterHandler (const wchar_t * __UNUSED_PARAM_1(expression),
+				 const wchar_t * __UNUSED_PARAM_1(function),
+				 const wchar_t * __UNUSED_PARAM_1(file),
+				 unsigned int    __UNUSED_PARAM_1(line),
+				 uintptr_t __UNUSED_PARAM(pReserved))
+{
+#ifdef __MINGW_SHOW_INVALID_PARAMETER_EXCEPTION
+  wprintf(L"Invalid parameter detected in function %s. File: %s Line: %d\n", function, file, line);
+  wprintf(L"Expression: %s\n", expression);
+#endif
+}
+
 static int __cdecl
 pre_c_init (void)
 {
@@ -121,7 +140,7 @@ pre_c_init (void)
 #else
   _setargv();
 #endif
-  if (_MINGW_INSTALL_DEBUG_MATHERR)
+  if (_MINGW_INSTALL_DEBUG_MATHERR == 1)
     {
       __setusermatherr (_matherr);
     }
@@ -263,17 +282,20 @@ __tmainCRTStartup (void)
 #ifdef _WIN64
     __mingw_init_ehandler ();
 #endif
-    __mingw_prepare_except_for_msvcr80_and_higher ();
+    _set_invalid_parameter_handler (__mingw_invalidParameterHandler);
     
     _fpreset ();
 
-    if (mingw_app_type)
-      {
+    __mingw_winmain_hInstance = (HINSTANCE) &__ImageBase;
+
 #ifdef WPRFLAG
-	lpszCommandLine = (_TCHAR *) _wcmdln;
+    lpszCommandLine = (_TCHAR *) _wcmdln;
 #else
-	lpszCommandLine = (char *) _acmdln;
+    lpszCommandLine = (char *) _acmdln;
 #endif
+
+    if (lpszCommandLine)
+      {
 	while (*lpszCommandLine > SPACECHAR || (*lpszCommandLine && inDoubleQuote))
 	  {
 	    if (*lpszCommandLine == DQUOTECHAR)
@@ -281,8 +303,8 @@ __tmainCRTStartup (void)
 #ifdef _MBCS
 	    if (_ismbblead (*lpszCommandLine))
 	      {
-		if (lpszCommandLine) /* FIXME: Why this check? Should I check for *lpszCommandLine != 0 too? */
-		  lpszCommandLine++;
+		if (lpszCommandLine[1])
+		  ++lpszCommandLine;
 	      }
 #endif
 	    ++lpszCommandLine;
@@ -290,8 +312,11 @@ __tmainCRTStartup (void)
 	while (*lpszCommandLine && (*lpszCommandLine <= SPACECHAR))
 	  lpszCommandLine++;
 
-	__mingw_winmain_hInstance = (HINSTANCE) &__ImageBase;
 	__mingw_winmain_lpCmdLine = lpszCommandLine;
+      }
+
+    if (mingw_app_type)
+      {
 	__mingw_winmain_nShowCmd = StartupInfo.dwFlags & STARTF_USESHOWWINDOW ?
 				    StartupInfo.wShowWindow : SW_SHOWDEFAULT;
       }
@@ -376,7 +401,7 @@ static void duplicate_ppstrings (int ac, wchar_t ***av)
 	avl=*av;
 	for (i=0; i < ac; i++)
 	  {
-		int l = wbytelen (avl[i]);
+		size_t l = wbytelen (avl[i]);
 		n[i] = (wchar_t *) malloc (l);
 		memcpy (n[i], avl[i], l);
 	  }
@@ -393,7 +418,7 @@ static void duplicate_ppstrings (int ac, char ***av)
 	avl=*av;
 	for (i=0; i < ac; i++)
 	  {
-		int l = strlen (avl[i]) + 1;
+		size_t l = strlen (avl[i]) + 1;
 		n[i] = (char *) malloc (l);
 		memcpy (n[i], avl[i], l);
 	  }
@@ -401,33 +426,3 @@ static void duplicate_ppstrings (int ac, char ***av)
 	*av = n;
 }
 #endif
-
-#ifdef __MINGW_SHOW_INVALID_PARAMETER_EXCEPTION
-#define __UNUSED_PARAM_1(x) x
-#else
-#define __UNUSED_PARAM_1	__UNUSED_PARAM
-#endif
-static void
-__mingw_invalidParameterHandler (const wchar_t * __UNUSED_PARAM_1(expression),
-				 const wchar_t * __UNUSED_PARAM_1(function),
-				 const wchar_t * __UNUSED_PARAM_1(file),
-				 unsigned int    __UNUSED_PARAM_1(line),
-				 uintptr_t __UNUSED_PARAM(pReserved))
-{
-#ifdef __MINGW_SHOW_INVALID_PARAMETER_EXCEPTION
-  wprintf(L"Invalid parameter detected in function %s. File: %s Line: %d\n", function, file, line);
-  wprintf(L"Expression: %s\n", expression);
-#endif
-}
-
-HANDLE __mingw_get_msvcrt_handle(void);
-
-static void __cdecl 
-__mingw_prepare_except_for_msvcr80_and_higher (void)
-{
-  _invalid_parameter_handler (*fIPH)(_invalid_parameter_handler) = NULL;
-
-  fIPH = (void*)GetProcAddress (__mingw_get_msvcrt_handle(), "_set_invalid_parameter_handler");
-  if (fIPH)
-    (*fIPH)(__mingw_invalidParameterHandler);
-}
