@@ -8,7 +8,7 @@
 
 char *gai_strerrorA(int ecode)
 {
-	DWORD dwMsgLen;
+	DWORD dwMsgLen __attribute__((unused));
 	static char buff[GAI_STRERROR_BUFFER_SIZE + 1];
 	dwMsgLen = FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS|FORMAT_MESSAGE_MAX_WIDTH_MASK,
 				  NULL, ecode, MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT), (LPSTR)buff,
